import React, { useState, useEffect, useMemo, useCallback } from "react";
import { Checkbox, FormLabel } from "@material-ui/core";
import MUIDataTable, { debounceSearchRender } from "mui-datatables";
import Skeleton from "@material-ui/lab/Skeleton";
import PageTitle from "../../../components/PageTitle";
import { useResourcesApi, useDictionariesApi } from "../../../api";
import { defaultResourcesFilters } from "../../../constants/resourcesConstants";

const __ROWS_PER_PAGE_OPTIONS = [10, 20, 50, 100];

const ResourcesContainer = () => {
  const [state, setState] = useState({
    pageSize: 10,
    totalCount: 0,
    values: []
  });
  const [loading, setLoading] = useState(false);
  const [filters, setFilters] = useState({ ...defaultResourcesFilters });
  const [resourceCategories, setResourceCategories] = useState([]);

  const { getResources } = useResourcesApi();
  const { getResourceCategories } = useDictionariesApi();

  useEffect(() => {
    getResourceCategories().then((r) => setResourceCategories(r));
  }, [getResourceCategories]);

  useEffect(() => {
    if (filters.loadedPages && filters.loadedPages.includes(filters.page))
      return;

    setLoading(true);
    getResources(filters, {
      onCompleted: (resources) => {
        setState((prev) => ({
          ...prev,
          ...resources,
          values: filters.loadedPages
            ? [...prev.values, ...resources.values]
            : resources.values
        }));

        setFilters((prev) => ({
          ...prev,
          loadedPages: prev?.loadedPages
            ? [...prev.loadedPages, resources.page]
            : [resources.page]
        }));

        setLoading(false);
      }
    });
  }, [getResources, filters]);

  const columns = useMemo(
    () => [
      {
        label: "Code",
        name: "resourceCode",
        options: {
          display: true,
          draggable: true,
          download: true,
          print: false,
          searchable: true,
          sort: true,
          customFilterListOptions: {
            render: (v) => (v ? `Code contains: ${v}` : [])
          },
          filter: true,
          filterType: "textField",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: undefined,
          hint: undefined
        }
      },
      {
        label: "Name",
        name: "resourceName",
        options: {
          display: true,
          draggable: true,
          download: true,
          print: false,
          searchable: true,
          sort: true,
          customFilterListOptions: {
            render: (v) => (v ? `Name contains: ${v}` : [])
          },
          filter: true,
          filterType: "textField",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: undefined,
          hint: undefined
        }
      },
      {
        label: "Category",
        name: "categoryName",
        options: {
          display: true,
          draggable: true,
          download: true,
          print: false,
          searchable: true,
          sort: true,
          filter: true,
          filterType: "dropdown",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false
          },
          filterList: [],
          hint: undefined
        }
      },
      {
        label: "Secured",
        name: "secured",
        options: {
          display: true,
          draggable: true,
          download: true,
          print: false,
          searchable: true,
          sort: true,
          customBodyRender: (value, _tableMeta, _updateValue) => {
            return (
              <Checkbox
                disabled
                size="small"
                checked={value}
                style={{ padding: "0" }}
              />
            );
          },
          customFilterListOptions: {
            render: (v) => {
              const checked = v[0];
              return checked ? <span>Secured</span> : [];
            }
          },
          filter: true,
          filterType: "custom",
          filterOptions: {
            logic: (_prop, _filterValue, _row) => false,
            display: (filterList, onChange, index, column) => {
              return (
                <div>
                  <FormLabel>Secured</FormLabel>
                  <Checkbox
                    color="primary"
                    checked={filterList[index][0] || false}
                    onChange={(event) => {
                      filterList[index][0] = event.target.checked;
                      onChange(filterList[index], index, column);
                    }}
                  />
                </div>
              );
            }
          },
          filterList: undefined,
          hint: undefined
        }
      }
    ],
    []
  );

  const changeFilters = useCallback((obj) => {
    const keys = Object.keys(obj);
    const isPageChange = keys.length === 1 && keys[0] === "page";
    if (!isPageChange) {
      const { page, loadedPages } = defaultResourcesFilters;
      obj = { ...obj, page, loadedPages };
    }

    setFilters((prev) => ({ ...prev, ...obj }));
  }, []);

  const handleFilterChange = useCallback(
    (changedColumn, filterList, type, changedColumnIndex, displayData) => {
      if (type === "reset") {
        changeFilters(defaultResourcesFilters);
        return;
      }
      const filterValue = filterList[changedColumnIndex][0];
      changeFilters({ [changedColumn]: filterValue });
    },
    [changeFilters]
  );

  return (
    <>
      <PageTitle title="Resources adminstration" />

      <MUIDataTable
        title="Resources"
        columns={columns}
        data={state.values}
        options={{
          filterType: "textField",
          expandableRows: false,
          print: false,
          selectableRows: "none",
          rowsPerPage: state.pageSize,
          rowsPerPageOptions: __ROWS_PER_PAGE_OPTIONS,
          count: state.totalCount,
          customSearchRender: debounceSearchRender(500),
          onChangePage: (currentPage) =>
            changeFilters({ page: currentPage + 1 }),
          onChangeRowsPerPage: (numberOfRows) =>
            changeFilters({ pageSize: numberOfRows }),
          onColumnSortChange: (changedColumn, direction) =>
            changeFilters({
              sortBy: changedColumn,
              sortDirection: direction
            }),
          onFilterChange: handleFilterChange,
          onSearchChange: (text) => changeFilters({ fullTextSearch: text }),
          customSort: (data) => data,
          customSearch: () => true,
          setFilterChipProps: (_colIndex, _colName, _data) => {
            return {
              color: "primary",
              variant: "outlined"
            };
          },
          textLabels: {
            body: {
              noMatch: loading ? (
                <Skeleton variant="text" />
              ) : (
                "Sorry, there is no matching data to display"
              )
            }
          }
        }}
      />
    </>
  );
};

export default ResourcesContainer;
